package com.yellowmessenger.flutter_xmpp;

import android.app.Activity;
import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import com.yellowmessenger.flutter_xmpp.sdk.MyBus;
import com.yellowmessenger.flutter_xmpp.sdk.events.MessageAcknowledgementEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.MessageReceivedEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.SendMessageEvent;
import com.yellowmessenger.flutter_xmpp.sdk.models.db.ChatMessage;
import com.yellowmessenger.flutter_xmpp.sdk.service.YellowMessenger;
import com.yellowmessenger.flutter_xmpp.sdk.utils.PreferencesManager;

import org.greenrobot.eventbus.EventBus;

import java.util.HashMap;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.reactivex.android.schedulers.AndroidSchedulers;

/** FlutterXmppPlugin */
public class FlutterXmppPlugin implements FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler {
  public static final String TAG = "eventchannel";
  public static final String STREAM = "com.yellowmessenger.bot/stream";
  private static final String CHANNEL = "com.yellowmessenger.bot/methods";
  private EventChannel.EventSink mEventSink;
  private Registrar mRegistrar;
  private Context mContext;
  private MethodChannel methodChannel;
  private EventChannel eventChannel;


  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
    onAttachedToEngine(binding.getApplicationContext(), binding.getFlutterEngine().getDartExecutor() );
  }

  public static void registerWith(Registrar registrar) {
    FlutterXmppPlugin plugin = new FlutterXmppPlugin();
    plugin.onAttachedToEngine(registrar.context(), registrar.messenger());
  }

  private void onAttachedToEngine(Context applicationContext, BinaryMessenger messenger) {
    this.mContext = applicationContext;
    methodChannel = new MethodChannel(messenger, CHANNEL);
    eventChannel = new EventChannel(messenger, STREAM);
    eventChannel.setStreamHandler(this);
    methodChannel.setMethodCallHandler(this);

  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    String msg = call.argument("msg");
    if (call.method.equals("sendData")) {
      try {
        Context context = mContext;
        PreferencesManager prefManager = PreferencesManager.getInstance(context);

        try{
          String username = prefManager.getAccount();;
          Log.d("MyUsername", username);
          String name = "CG_V0";
          ChatMessage chatMessage = new ChatMessage(username, msg, name, true);
          chatMessage.setMessageValue(msg);
          Log.d("Priyank", "Sending - " + msg);
          EventBus.getDefault().post(new SendMessageEvent(chatMessage));
          result.success("Sending Message...");
        }
        catch (Exception e){
          Log.e(TAG, e.getMessage());
        }

      }
      catch (Exception e){
        Log.e(TAG, e.getMessage());
      }
    }

    if (call.method.equals("initializeXMPP")) {
      String botId = call.argument("botId");
      String botName = call.argument("botName");
      String authToken = call.argument("authToken");
      if(mContext == null) Log.d("Priyank", "Context not set.");
      initXMPP(mContext, botId, botName, authToken);
      result.success(true);
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
  }

  @Override
  public void onListen(Object arguments, EventChannel.EventSink events) {
    Log.w(TAG, "adding listener");
    mEventSink = events;
    try {

      MyBus.getInstance().bus().toObservable()
              .observeOn(AndroidSchedulers.mainThread())
              .subscribe(object -> {
                        if (object instanceof MessageReceivedEvent) {
                          Log.d("Priyank", "Event Recieved");
                          mEventSink.success(((MessageReceivedEvent) object).getChatMessage().getMessage());
                          //mEventSink.success(((MessageReceivedEvent) object).getChatMessage().getBitmap());
                        }
                      }
              );

      MyBus.getInstance().bus().toObservable()
              .observeOn(AndroidSchedulers.mainThread())
              .subscribe(object -> {
                        if (object instanceof MessageAcknowledgementEvent) {
                          Log.d("Priyank", ((MessageAcknowledgementEvent) object).getChatMessage().getMessage());
                          // mEventSink.success(((MessageAcknowledgementEvent) object).getChatMessage().getMessage());
                        }
                      }
              );
    }
    catch (Exception e){
      Log.e(TAG, e.getMessage());
    }
  }

  @Override
  public void onCancel(Object args) {
    Log.w(TAG, "cancelling listener");
  }

  private void initXMPP(Context context,String botId, String botName, String authToken) {
    try {
      HashMap<String, String> props = new HashMap<>();
      props.put("account", botId);
      props.put("name", botName);
      props.put("authorizationToken", authToken);
      YellowMessenger.init(context, props);
      mEventSink.success("Initializing...");
    }
    catch (NullPointerException e){
      Log.d("Priyank", e.getMessage());
    }
  }


}
