package com.yellowmessenger.flutter_xmpp.sdk;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.yellowmessenger.flutter_xmpp.sdk.dao.ChatMessageDAO;
import com.yellowmessenger.flutter_xmpp.sdk.events.ChatConnectedEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.ChatDisconnectedEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.MessageAcknowledgementEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.MessageReceivedEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.SendActionEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.SendMessageEvent;
import com.yellowmessenger.flutter_xmpp.sdk.events.SendOptionEvent;
import com.yellowmessenger.flutter_xmpp.sdk.models.ChatType;
import com.yellowmessenger.flutter_xmpp.sdk.models.Option;
import com.yellowmessenger.flutter_xmpp.sdk.models.Question;
import com.yellowmessenger.flutter_xmpp.sdk.models.db.ChatMessage;
import com.yellowmessenger.flutter_xmpp.sdk.receivers.UploadReceiver;
import com.yellowmessenger.flutter_xmpp.sdk.utils.PreferencesManager;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


public class ChatActivity extends AppCompatActivity  implements GoogleApiClient.OnConnectionFailedListener {


    UploadReceiver uploadReceiver;

    public void sendMessage(String message) {
        if (!message.trim().equals("")) {
            ChatMessage chatMessage = new ChatMessage(username, message, name, true);
            EventBus.getDefault().post(new SendMessageEvent(chatMessage));
        }
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EventBus.getDefault().register(this);
        uploadReceiver = new UploadReceiver();
        chatMessages = new ArrayList<>();
        // set listeners
        init(getIntent());
    }


    String language = "EN";
    ProgressDialog dialog;


    @Subscribe
    public void onEvent(final MessageReceivedEvent event) {
        if(event.getChatMessage().getChatType() != ChatType.EVENT){
           // addMessage(event.getChatMessage());
        } else if(event.getChatMessage().getChatResponse().getEvent().getCode() != null && event.getChatMessage().getChatResponse().getEvent().getCode().equals("load_pdf")){

        }

        }

    private String username;
    private String name;
    private List<ChatMessage> chatMessages;

    private void init(Intent intent) {
        String oldUsername = username;
        username = intent.getExtras().getString("username");
        name = intent.getExtras().getString("name");


        // Save the name
        PreferencesManager.getInstance(getApplicationContext()).setBusinessName(username,name);
        setTitle(name);
        if (oldUsername == null || !username.equals(oldUsername)) {
            getChatHistory();
        }
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        setIntent(intent);
        init(intent);
    }


    private void getChatHistory() {
        chatMessages.clear();
        List<ChatMessage> newChatMessages = ChatMessageDAO.findAllByUsername(username, 200, 0);
        if(newChatMessages.size()==0){
            ChatMessage chatMessage = new ChatMessage(username, "Start", name, true);
            chatMessage.setMessageValue("GET_STARTED");
            EventBus.getDefault().post(new SendMessageEvent(chatMessage));
        }
        Collections.reverse(newChatMessages);
        chatMessages.addAll(newChatMessages);
    }

    private void getChatNewHistory() {
        List<ChatMessage> newChatMessages = ChatMessageDAO.findAllByUsernameAndIdGreaterThan(username,getLastId());
        chatMessages.addAll(newChatMessages);
    }

    private long getLastId(){
        if(chatMessages.size()>0){
            for(int i =chatMessages.size()-1; i>=0; i-- ){
                if(chatMessages.get(i).getId()!=null){
                    return chatMessages.get(i).getId();
                }
            }
        }

        return 0;
    }

    @Override
    protected void onPostResume() {
        super.onPostResume();
        getChatNewHistory();
        EventBus.getDefault().post(new ChatConnectedEvent(username));
    }

    @Override
    public void onStop() {
        EventBus.getDefault().post(new ChatDisconnectedEvent(username));
        super.onStop();
    }

    @Override
    protected void onPause() {
        EventBus.getDefault().post(new ChatDisconnectedEvent(username));
        super.onPause();
    }



    private void clearLog(){
        this.chatMessages.clear();
        ChatMessageDAO.deletAllByUesername(username);
    }


    public void sendActionEvent(SendActionEvent sendActionEvent) {
        ChatMessage chatMessage = new ChatMessage(username, sendActionEvent.getAction().getTitle(), name, true);
        //addMessage(chatMessage);
        chatMessage.setMessageValue(sendActionEvent.getAction().getText()!=null?sendActionEvent.getAction().getText():sendActionEvent.getAction().getTitle());
        EventBus.getDefault().post(new SendMessageEvent(chatMessage));
    }

    public void sendOptionEvent(SendOptionEvent sendOptionEvent) {

        ChatMessage chatMessage = new ChatMessage(username, sendOptionEvent.getOption().getLabel(), name, true);
        //addMessage(chatMessage);
        chatMessage.setMessageValue(sendOptionEvent.getOption().getValue());
        EventBus.getDefault().post(new SendMessageEvent(chatMessage));
    }



    @Subscribe
    public void onEvent(MessageAcknowledgementEvent event) {
        ChatMessage chatMessage = event.getChatMessage();
        if (chatMessage.getUsername() != null && chatMessage.getUsername().equals(username)) {
            for (ChatMessage message : chatMessages) {
                if ((message.getStanzaId() != null && message.getStanzaId().equals(chatMessage.getStanzaId())) || (message.getId() != null && message.getId().equals(chatMessage.getId()))) {
                    message.setAcknowledged(chatMessage.getAcknowledged());
                }
            }

        }
    }





    public void sendOption(final Question question, final Option option) {
        if (option.isLocation()) {
           // askForLocation();
        } else {

            ChatMessage chatMessage = new ChatMessage(username, option.getLabel(), name, true);
            chatMessage.setMessageValue(option.getValue());
            EventBus.getDefault().post(new SendMessageEvent(chatMessage));
           // addMessage(chatMessage);
        }
    }







    @Override
    protected void onDestroy() {
        EventBus.getDefault().unregister(this);
        //mGoogleApiClient.disconnect();
        //tts.shutdown();
        super.onDestroy();
    }

    @Override
    public void onConnectionFailed(@NonNull ConnectionResult connectionResult) {

    }
}

